package eu.ztsh.garmin

import android.Manifest
import android.annotation.SuppressLint
import android.app.PendingIntent
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothManager
import android.bluetooth.BluetoothSocket
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.widget.CompoundButton
import androidx.activity.result.ActivityResultCallback
import androidx.activity.result.contract.ActivityResultContracts.StartActivityForResult
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import eu.ztsh.garmin.databinding.ActivityMainBinding
import me.trevi.navparser.lib.NavigationData
import me.trevi.navparser.service.*
import java.io.IOException
import java.util.*

@SuppressLint("MissingPermission")
class MainActivity : AppCompatActivity() {

    var navigating: Boolean = false
    lateinit var garmin: Garmin
    private lateinit var binding : ActivityMainBinding
    private val navDataModel: NavigationDataModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)
        bluetoothInit()

        binding.enabledSwitch.setOnCheckedChangeListener {
            _, isChecked ->
            run {
                navigating = isChecked

        setNavigationData(NavigationData(true))
        navDataModel.liveData.observe(this) { setNavigationData(it) }
                Log.d(TAG, "OK")
            }
        }
    }

    override fun onStart() {
        super.onStart()

        startServiceListener()
        checkNotificationsAccess()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, intent: Intent?) {
        super.onActivityResult(requestCode, resultCode, intent)

        Log.d(TAG, "Got activity result: $intent, ${intent?.extras}, ${intent?.action}")
        when (intent?.action) {

            NOTIFICATIONS_ACCESS_RESULT -> {
                intent.getBooleanExtra(NOTIFICATIONS_ACCESS_RESULT, false).also {
                    val notificationAccess = it

                    if (!notificationAccess) {
                        Log.e(TAG, "No notification access for ${NavigationListenerEmitter::class.qualifiedName}")
                    }
                }
            }

            NAVIGATION_DATA_UPDATED -> {
//                gotoFragment(R.id.NavigationFragment)

                val navData = intent.getParcelableExtra<NavigationData>(NAVIGATION_DATA)

                Log.d(TAG, "Got navigation data $navData")
                navDataModel.data = navData
            }

            NAVIGATION_STARTED -> {
                Log.d(TAG, "Started")
            }

            NAVIGATION_STOPPED -> {
                Log.d(TAG, "Stopped")
            }
        }
    }

    private fun bluetoothInit() {
        val bluetoothManager: BluetoothManager = getSystemService(BluetoothManager::class.java)
        val bluetoothAdapter: BluetoothAdapter = bluetoothManager.adapter
            ?: // Device doesn't support Bluetooth
            throw Exception()
        if (!bluetoothAdapter.isEnabled) {
            // TODO: Start intent
            val enableBtIntent = Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE)
            registerForActivityResult(StartActivityForResult(), ActivityResultCallback { })
        }
        checkBt()

        val pairedDevices: Set<BluetoothDevice>? = bluetoothAdapter.bondedDevices
        val context = this
        pairedDevices?.firstOrNull { device ->
            Log.d(TAG, device.name)
            device.name.equals("GARMIN HUD")
        }?.apply {
            garmin = Garmin(context, this, bluetoothAdapter)
            garmin.start()
        }

    }

    fun checkBt(): Boolean {
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.S) {
            checkBt(Manifest.permission.BLUETOOTH_SCAN)
            checkBt(Manifest.permission.BLUETOOTH_CONNECT)
        } else {
            checkBt(Manifest.permission.BLUETOOTH)
            checkBt(Manifest.permission.BLUETOOTH_ADMIN)
        }
        return true
    }

    private fun checkBt(permission: String): Boolean {
        if (ActivityCompat.checkSelfPermission(
                this,
                permission
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(this, arrayOf(permission), 1)
        }
        return true
    }

    private fun startServiceListener() {
        Log.d(TAG, "Start Service Listener")
        setServiceListenerIntent(createPendingResult(100, Intent(), 0))
    }

    fun stopServiceListener() {
        Log.d(TAG, "Stopping Service Listener")
        setServiceListenerIntent(null)
    }

    private fun setServiceListenerIntent(pendingIntent: PendingIntent?) {
        startService(serviceIntent(SET_INTENT).putExtra(PENDING_INTENT, pendingIntent))
    }

    private fun checkNotificationsAccess() {
        startService(serviceIntent(CHECK_NOTIFICATIONS_ACCESS))
    }

    private fun serviceIntent(action: String): Intent {
        return Intent(applicationContext, NavigationListenerEmitter::class.java).setAction(action)
    }
    private fun setNavigationData(navData: NavigationData) {
        Log.d(TAG, "Sending $navData")
        garmin.send(navData)
    }

    companion object {
        private const val TAG = "bt"
    }

}
